/**
 * @file misc.js
 * @description 杂项
 */
const KIT_COM_MISC = (function() {

  function signature() {
    return 'KIT_COM_MISC';
  }

  const out = {
    signature
  };

  return out;

})();

/**
 * @file fs.js
 * @description 文件系统相关
 */
const KIT_FS_FS = (function() {

  function signature() {
    return 'KIT_FS_FS';
  }

  const fs = require("fs");
  const Path = require("path");

  /**
   * @function calcFileMd5
   * 计算文件md5, 支持大文件
   */
  function calcFileMd5(file) {
    var md5 = "";
    if (!fs.existsSync(file)) {
      console.error("not found the file: " + file);
      return md5;
    }

    var hash = require("crypto").createHash("md5");
    var fd = fs.openSync(file, "r");
    var buf = new Buffer(64 * 1024);
    for (var rd = 0; (rd = fs.readSync(fd, buf, 0, buf.length)) > 0; ) {
      hash.update(buf.slice(0, rd));
      //console.log("read size: " + rd);
    }
    fs.closeSync(fd);
    md5 = hash.digest("hex");
    //console.log("read file END!! md5=" + md5);
    return md5;
  }


  /**
   * 文件信息类
   */
  function FileInfo(path, states) {
    this.path = path;					// name with path
    this.name = Path.basename(path);	// name only
    this.size = 0;	// size
    if (null != states) {
      this.size = states.size;
      this.mtime = states.mtime;
    }
    this._id  = calcFileMd5(this.path);
  }
  FileInfo.prototype.info = function() {
    var s = ("path=" + this.path
            + "; name=" + this.name
            + "; size=" + (this.size / 1024).toFixed(2) + "KB"
            + "; md5=" + this._id
            + "; mtime=" + this.mtime);
    return s;
  }
  FileInfo.prototype.toString = function() {
    var s = JSON.stringify(this);
    return s;
  }


  /**
   * 列表序列化到string
   */
  function flist2string(flist) {
    var s = flist.join('\n');
    return s;
  }

  /**
   * get file list
   */
  function getFileList(path) {
    var flist = [];
    readFile(path, flist);
    return flist;
  }

  // 遍历读取文件
  function readFile(path, flist) {
    var files = fs.readdirSync(path);
    files.forEach(walk);
    function walk(file) {
      var fpath = Path.join(path, file);
      try {
        var states = fs.statSync(fpath);
          if (states.isDirectory()) {
            readFile(fpath, flist);	// 递归
          } else {
            var f = new FileInfo(fpath, states);
            // console.log("add: " + f.toString());
            flist.push(f);
          }
      } catch (e) {
        console.error("error excption: %s - %s", fpath, e.message);
      }
    }
  }

  // write utf-8 buffer string to file
  function writeFile(file, data) {
    fs.writeFileSync(file, data, "utf-8");
    console.log("file wrote finished!!");
  }

  // file sort handler
  function sortHandlerBySize(lhs, rhs) {
    if (lhs.size > rhs.size) {
      return -1;
    } else if (lhs.size < rhs.size) {
      return 1;
    } else {
      return 0;
    }
  }


  function calcDirMd5(indir, outfile) {
    var flist = getFileList(indir);
    flist.sort(sortHandlerBySize);
    writeFile(outfile, flist2string(flist));
    console.log("total process time: " + process.uptime());
    return 0;
  }

  function showUsage() {
    var jsname = Path.basename(process.argv[1]);
    console.log("Usage: node %s <indir> <outfile>", jsname);
  }


  /**
   * main入口
   */
  function main(argv) {
    if (argv.length < 2) {
      showUsage();
      return 2;
    }
    var indir = argv[0];
    var outfile = argv[1];
    console.log("indir=%s\noutfile=%s", indir, outfile);
    var ret = run(indir, outfile);
    process.exit(ret);
  }

  const out = {
    signature,
    getFileList,
    calcFileMd5,
    calcDirMd5
  };

  return out;

})();

/**
 * @file http.js
 * @description url相关封装
 */

const KIT_RPC_HTTP = (function () {

  var urlparse = require('url').parse;
  var http = require('http');
  var fs = require('fs');

  /**
   * @function UrlUpload
   * @description url upload by POST
   */
  function UrlUpload(url, uploadfile, callback) {
    var urlinfo = urlparse(url);
    var options = {
      method: 'POST',
      host: urlinfo.hostname,
      path: urlinfo.pathname
    };
    if (urlinfo.port) {
      options.port = urlinfo.port;
    }
    if (urlinfo.search) {
      options.path += urlinfo.search;
    }
    var req = http.request(options, function (res) {
      var chunks = [], length = 0;
      res.on('data', function (chunk) {
        length += chunk.length;
        chunks.push(chunk);
      });
      res.on('end', function () {
        var buffer = new Buffer(length);
        // delay copy
        for (var i = 0, pos = 0, size = chunks.length; i < size; i++) {
          chunks[i].copy(buffer, pos);
          pos += chunks[i].length;
        }
        res.body = buffer;
        callback(null, res);
      });
    });
    var readstream = fs.createReadStream(uploadfile);
    readstream.on('data', function (chunk) {
      console.log('write', chunk.length);
      req.write(chunk);
    });
    readstream.on('end', function () {
      req.end();
    });
  }


  /**
   * @function UrlDownload
   * @description url download by GET
   */
  function UrlDownload(url, savefile, callback = UrlOnError) {
    console.log('download', url, 'to', savefile);
    var urlinfo = urlparse(url);
    var options = {
      method: 'GET',
      host: urlinfo.hostname,
      path: urlinfo.pathname
    };
    if (urlinfo.port) {
      options.port = urlinfo.port;
    }
    if (urlinfo.search) {
      options.path += urlinfo.search;
    }
    var req = http.request(options, function (res) {
      var writestream = fs.createWriteStream(savefile);
      writestream.on('close', function () {
        callback(null, res);
      });
      res.pipe(writestream);
    });
    req.end();
  }

  function UrlOnError(err, res) {
    console.log(res.statusCode, res.headers);
  }

  const out = {
    UrlUpload,
    UrlDownload,
    UrlOnError
  };

  return out;

})();


/**
 * @file ws.js
 * @description web-socket
 */

const KIT_RPC_WS = (function() {

  class WSClient {

  };

  const out = {
    WSClient
  };

  return out;
})();

/**
 * @file nbox-kit-index.js
 * @description 
 */

const KIT = Object.assign(
  {},
  KIT_COM_MISC,
  KIT_FS_FS,
  KIT_RPC_HTTP,
  KIT_RPC_WS
);

KIT.help = () => { console.log('kit: ', module.exports); }

module.exports = KIT;
